@@ -13,7 +13,7 @@ class TestPost < Test::Unit::TestCase
   context "A Post" do
     setup do
       clear_dest
-      stub(Jekyll).configuration { Jekyll::DEFAULTS }
+      stub(Jekyll).configuration { Jekyll::Configuration::DEFAULTS }
       @site = Site.new(Jekyll.configuration)
     end
 
@@ -25,6 +25,34 @@ class TestPost < Test::Unit::TestCase
       assert !Post.valid?("blah")
     end
 
+    should "make properties accessible through #[]" do
+      post = setup_post('2013-12-20-properties.text')
+
+      attrs = {
+        categories: %w(foo bar baz),
+        content: "All the properties.\n\nPlus an excerpt.\n",
+        date: Time.new(2013, 12, 20),
+        dir: "/foo/bar/baz/2013/12/20",
+        excerpt: "All the properties.\n\n",
+        foo: 'bar',
+        id: "/foo/bar/baz/2013/12/20/properties",
+        layout: 'default',
+        name: nil,
+        path: "_posts/2013-12-20-properties.text",
+        permalink: nil,
+        published: nil,
+        tags: %w(ay bee cee),
+        title: 'Properties Post',
+        url: "/foo/bar/baz/2013/12/20/properties.html"
+      }
+
+      attrs.each do |attr, val|
+        attr_str = attr.to_s
+        result = post[attr_str]
+        assert_equal val, result, "For <post[\"#{attr_str}\"]>:"
+      end
+    end
+
     context "processing posts" do
       setup do
         @post = Post.allocate
@@ -58,21 +86,39 @@ class TestPost < Test::Unit::TestCase
         end
       end
 
-      should "CGI escape urls" do
+      should "escape urls" do
         @post.categories = []
         @post.process("2009-03-12-hash-#1.markdown")
         assert_equal "/2009/03/12/hash-%231.html", @post.url
         assert_equal "/2009/03/12/hash-#1", @post.id
       end
 
+      should "escape urls with non-alphabetic characters" do
+        @post.categories = []
+        @post.process("2014-03-22-escape-+ %20[].markdown")
+        assert_equal "/2014/03/22/escape-+%20%2520%5B%5D.html", @post.url
+        assert_equal "/2014/03/22/escape-+ %20[]", @post.id
+      end
+
       should "respect permalink in yaml front matter" do
         file = "2008-12-03-permalinked-post.textile"
         @post.process(file)
         @post.read_yaml(@source, file)
 
         assert_equal "my_category/permalinked-post", @post.permalink
-        assert_equal "my_category", @post.dir
-        assert_equal "my_category/permalinked-post", @post.url
+        assert_equal "/my_category", @post.dir
+        assert_equal "/my_category/permalinked-post", @post.url
+      end
+
+      should "not be writable outside of destination" do
+        unexpected = File.expand_path("../../../baddie.html", dest_dir)
+        File.delete unexpected if File.exist?(unexpected)
+        post = setup_post("2014-01-06-permalink-traversal.md")
+        do_render(post)
+        post.write(dest_dir)
+
+        assert !File.exist?(unexpected)
+        assert File.exist?(File.expand_path("baddie.html", dest_dir))
       end
 
       context "with CRLF linebreaks" do
@@ -88,6 +134,18 @@ class TestPost < Test::Unit::TestCase
         end
       end
 
+      context "with three dots ending YAML header" do
+        setup do
+          @real_file = "2014-03-03-yaml-with-dots.md"
+        end
+        should "should read the YAML header" do
+          @post.read_yaml(@source, @real_file)
+
+          assert_equal({"title" => "Test Post Where YAML Ends in Dots"},
+                       @post.data)
+        end
+      end
+
       context "with embedded triple dash" do
         setup do
           @real_file = "2010-01-08-triple-dash.markdown"
@@ -128,6 +186,30 @@ class TestPost < Test::Unit::TestCase
           end
         end
 
+        context "with unspecified (date) style and a numeric category" do
+          setup do
+            @post.categories << 2013
+            @post.process(@fake_file)
+          end
+
+          should "process the url correctly" do
+            assert_equal "/:categories/:year/:month/:day/:title.html", @post.template
+            assert_equal "/2013/2008/09/09/foo-bar.html", @post.url
+          end
+        end
+
+        context "with specified layout of nil" do
+          setup do
+            file = '2013-01-12-nil-layout.textile'
+            @post = setup_post(file)
+            @post.process(file)
+          end
+
+          should "layout of nil is respected" do
+            assert_equal "nil", @post.data["layout"]
+          end
+        end
+
         context "with unspecified (date) style and categories" do
           setup do
             @post.categories << "food"
@@ -141,6 +223,19 @@ class TestPost < Test::Unit::TestCase
           end
         end
 
+        context "with space (categories)" do
+          setup do
+            @post.categories << "French cuisine"
+            @post.categories << "Belgian beer"
+            @post.process(@fake_file)
+          end
+
+          should "process the url correctly" do
+            assert_equal "/:categories/:year/:month/:day/:title.html", @post.template
+            assert_equal "/French%20cuisine/Belgian%20beer/2008/09/09/foo-bar.html", @post.url
+          end
+        end
+
         context "with none style" do
           setup do
             @post.site.permalink_style = :none
@@ -165,6 +260,18 @@ class TestPost < Test::Unit::TestCase
           end
         end
 
+        context "with ordinal style" do
+          setup do
+            @post.site.permalink_style = :ordinal
+            @post.process(@fake_file)
+          end
+
+          should "process the url correctly" do
+            assert_equal "/:categories/:year/:y_day/:title.html", @post.template
+            assert_equal "/2008/253/foo-bar.html", @post.url
+          end
+        end
+
         context "with custom date permalink" do
           setup do
             @post.site.permalink_style = '/:categories/:year/:i_month/:i_day/:title/'
@@ -176,6 +283,17 @@ class TestPost < Test::Unit::TestCase
           end
         end
 
+        context "with custom abbreviated month date permalink" do
+          setup do
+            @post.site.permalink_style = '/:categories/:year/:short_month/:day/:title/'
+            @post.process(@fake_file)
+          end
+
+          should "process the url correctly" do
+            assert_equal "/2008/Sep/09/foo-bar/", @post.url
+          end
+        end
+
         context "with prefix style and no extension" do
           setup do
             @post.site.permalink_style = "/prefix/:title"
@@ -203,12 +321,77 @@ class TestPost < Test::Unit::TestCase
 
         assert_equal "<h1>{{ page.title }}</h1>\n<p>Best <strong>post</strong> ever</p>", @post.content
       end
+
+      context "#excerpt" do
+        setup do
+          file = "2013-01-02-post-excerpt.markdown"
+          @post = setup_post(file)
+          @post.process(file)
+          @post.read_yaml(@source, file)
+          do_render(@post)
+        end
+
+        should "return first paragraph by default" do
+          assert @post.excerpt.include?("First paragraph"), "contains first paragraph"
+          assert !@post.excerpt.include?("Second paragraph"), "does not contains second paragraph"
+          assert !@post.excerpt.include?("Third paragraph"), "does not contains third paragraph"
+        end
+
+        should "correctly resolve link references" do
+          assert @post.excerpt.include?("www.jekyllrb.com"), "contains referenced link URL"
+        end
+
+        should "return rendered HTML" do
+          assert_equal "<p>First paragraph with <a href=\"http://www.jekyllrb.com/\">link ref</a>.</p>\n\n",
+                       @post.excerpt
+        end
+
+        context "with excerpt_separator setting" do
+          setup do
+            file = "2013-01-02-post-excerpt.markdown"
+
+            @post.site.config['excerpt_separator'] = "\n---\n"
+
+            @post.process(file)
+            @post.read_yaml(@source, file)
+            @post.transform
+          end
+
+          should "respect given separator" do
+            assert @post.excerpt.include?("First paragraph"), "contains first paragraph"
+            assert @post.excerpt.include?("Second paragraph"), "contains second paragraph"
+            assert !@post.excerpt.include?("Third paragraph"), "does not contains third paragraph"
+          end
+
+          should "replace separator with new-lines" do
+            assert !@post.excerpt.include?("---"), "does not contains separator"
+          end
+        end
+
+        context "with custom excerpt" do
+          setup do
+            file = "2013-04-11-custom-excerpt.markdown"
+            @post = setup_post(file)
+            do_render(@post)
+          end
+
+          should "use custom excerpt" do
+            assert_equal("I can set a custom excerpt", @post.excerpt)
+          end
+
+          should "expose custom excerpt to liquid" do
+            assert @post.content.include?("I can use the excerpt: <quote>I can set a custom excerpt</quote>"), "Exposes incorrect excerpt to liquid."
+          end
+
+        end
+
+      end
     end
 
     context "when in a site" do
       setup do
         clear_dest
-        stub(Jekyll).configuration { Jekyll::DEFAULTS }
+        stub(Jekyll).configuration { Jekyll::Configuration::DEFAULTS }
         @site = Site.new(Jekyll.configuration)
         @site.posts = [setup_post('2008-02-02-published.textile'),
                        setup_post('2009-01-27-categories.textile')]
@@ -232,16 +415,6 @@ class TestPost < Test::Unit::TestCase
     end
 
     context "initializing posts" do
-      should "publish when published yaml is no specified" do
-        post = setup_post("2008-02-02-published.textile")
-        assert_equal true, post.published
-      end
-
-      should "not published when published yaml is false" do
-        post = setup_post("2008-02-02-not-published.textile")
-        assert_equal false, post.published
-      end
-
       should "recognize date in yaml" do
         post = setup_post("2010-01-09-date-override.textile")
         do_render(post)
@@ -299,6 +472,12 @@ class TestPost < Test::Unit::TestCase
         assert_equal [], post.categories
       end
 
+      should "recognize number category in yaml" do
+        post = setup_post("2013-05-10-number-category.textile")
+        assert post.categories.include?('2013')
+        assert !post.categories.include?(2013)
+      end
+
       should "recognize tag in yaml" do
         post = setup_post("2009-05-18-tag.textile")
         assert post.tags.include?('code')
@@ -310,7 +489,7 @@ class TestPost < Test::Unit::TestCase
         assert post.tags.include?('cooking')
         assert post.tags.include?('pizza')
       end
-      
+
       should "recognize empty tag in yaml" do
         post = setup_post("2009-05-18-empty-tag.textile")
         assert_equal [], post.tags
@@ -348,7 +527,27 @@ class TestPost < Test::Unit::TestCase
           post.write(dest_dir)
 
           assert File.directory?(dest_dir)
-          assert File.exists?(File.join(dest_dir, '2008', '10', '18', 'foo-bar.html'))
+          assert File.exist?(File.join(dest_dir, '2008', '10', '18', 'foo-bar.html'))
+        end
+
+        should "write properly when url has hash" do
+          post = setup_post("2009-03-12-hash-#1.markdown")
+          do_render(post)
+          post.write(dest_dir)
+
+          assert File.directory?(dest_dir)
+          assert File.exist?(File.join(dest_dir, '2009', '03', '12',
+                                        'hash-#1.html'))
+        end
+
+        should "write properly when url has space" do
+          post = setup_post("2014-03-22-escape-+ %20[].markdown")
+          do_render(post)
+          post.write(dest_dir)
+
+          assert File.directory?(dest_dir)
+          assert File.exist?(File.join(dest_dir, '2014', '03', '22',
+                                        'escape-+ %20[].html'))
         end
 
         should "write properly without html extension" do
@@ -358,7 +557,7 @@ class TestPost < Test::Unit::TestCase
           post.write(dest_dir)
 
           assert File.directory?(dest_dir)
-          assert File.exists?(File.join(dest_dir, 'foo-bar', 'index.html'))
+          assert File.exist?(File.join(dest_dir, 'foo-bar', 'index.html'))
         end
 
         should "insert data" do
@@ -373,7 +572,7 @@ class TestPost < Test::Unit::TestCase
           post.site.source = File.join(File.dirname(__FILE__), 'source')
           do_render(post)
 
-          assert_equal "<<< <hr />\n<p>Tom Preston-Werner github.com/mojombo</p>\n\n<p>This <em>is</em> cool</p> >>>", post.output
+          assert_equal "<<< <hr />\n<p>Tom Preston-Werner\ngithub.com/mojombo</p>\n\n<p>This <em>is</em> cool</p>\n >>>", post.output
         end
 
         should "render date specified in front matter properly" do
@@ -398,46 +597,53 @@ class TestPost < Test::Unit::TestCase
       assert_equal ['foo'], post.categories
     end
   end
-  
+
   context "converter file extension settings" do
     setup do
-      stub(Jekyll).configuration { Jekyll::DEFAULTS }
+      stub(Jekyll).configuration { Jekyll::Configuration::DEFAULTS }
       @site = Site.new(Jekyll.configuration)
     end
-    
+
     should "process .md as markdown under default configuration" do
       post = setup_post '2011-04-12-md-extension.md'
       conv = post.converter
-      assert conv.kind_of? Jekyll::MarkdownConverter
+      assert conv.kind_of? Jekyll::Converters::Markdown
     end
-    
-    should "process .text as indentity under default configuration" do
+
+    should "process .text as identity under default configuration" do
       post = setup_post '2011-04-12-text-extension.text'
       conv = post.converter
-      assert conv.kind_of? Jekyll::IdentityConverter
+      assert conv.kind_of? Jekyll::Converters::Identity
     end
-    
+
     should "process .text as markdown under alternate configuration" do
       @site.config['markdown_ext'] = 'markdown,mdw,mdwn,md,text'
       post = setup_post '2011-04-12-text-extension.text'
       conv = post.converter
-      assert conv.kind_of? Jekyll::MarkdownConverter
+      assert conv.kind_of? Jekyll::Converters::Markdown
     end
-    
+
     should "process .md as markdown under alternate configuration" do
       @site.config['markdown_ext'] = 'markdown,mkd,mkdn,md,text'
       post = setup_post '2011-04-12-text-extension.text'
       conv = post.converter
-      assert conv.kind_of? Jekyll::MarkdownConverter
+      assert conv.kind_of? Jekyll::Converters::Markdown
     end
-    
+
+    should "process .mkdn under text if it is not in the markdown config" do
+      @site.config['markdown_ext'] = 'markdown,mkd,md,text'
+      post = setup_post '2013-08-01-mkdn-extension.mkdn'
+      conv = post.converter
+      assert conv.kind_of? Jekyll::Converters::Identity
+    end
+
     should "process .text as textile under alternate configuration" do
       @site.config['textile_ext'] = 'textile,text'
       post = setup_post '2011-04-12-text-extension.text'
       conv = post.converter
-      assert conv.kind_of? Jekyll::TextileConverter
+      assert conv.kind_of? Jekyll::Converters::Textile
     end
-    
+
   end
-  
+
 end
